from pprint import *
from ReadKnowledgeBase import *
from CreateOntology import *

# Example list of attributes: 
#  The first argument is the attribute's name,
#  The second argument is the percentage,
#  The third argument is a boolean (isHead)
#   which explain if the attribute belongs 
#   to the head or to the modifier concept.
evil_chair = [("Demoniac",0.9,True),
        ("Opponent Hero",0.75,True),
        ("Protagonist",0.75,True),
        ("Impulsive",0.8,True),
        ("Has Back",0.95,False),
        ("Wood",0.65,False),
        ("Comfortable",0.8,False),
        ("Inflammable",0.7,False)]

ex_t = [("attr2",0.5,True), ("attr1",0.5,True)]
ex_not_t = [("-attr1",True)]


# Class Table
# L'obiettivo di questa classe e' di creare una tabella
#  che, data una lista con attributi e relative percentuali,
#  ne calcola le "combinazioni" e le inserisce nella tabella.
class Table:

    # Costruttore
    #  Salva la lista di attributi nel parametro tipical_attrs.
    #  Crea una "tabella di verità" (fatta di combinazioni di 0 e 1)
    #  Agginge alla tabella una colonna con ogni percentuale 
    #   calcolata sulla linea
    def __init__(self, from_input) :
        
        self.title = from_input.title
        self.h_conc = from_input.head_conc
        self.m_conc = from_input.mod_conc

        self.attrs = from_input.attrs
        self.tipical_attrs = from_input.tipical_attrs
        self.table = self.createTable()
        self.addPercentage()
        self.delNotConsistent()
        self.sorted_table = self.getSortedTable()
        #print ("\n\n ----- TABELLA ORDINATA ------ \n")
        #self.printTable()
        #print ("\n\n ----------------------------- \n")

    # Metodo per creare le nostre tabelle di 0 e 1
    #  L'intuizione e' che nelle tabelle di verità, leggendo 
    #   ogni riga si possono leggere dei numeri binari;
    #   quindi basta riempire ogni riga con 0 e 1 corrispondenti
    #   al numero in binario della riga (partendo dalla riga 0)
    def createTable(self) :
        w, h = len(self.tipical_attrs), pow(2, len(self.tipical_attrs))
        m = [self.to_binary(y, w) for y in range(h)]
        return m

    # Metodo per calcolare il numero binario dato un numero
    #  in base 10
    # NB! se il numero occupa meno cifre del numero di colonne 
    #  che dovrà avere la tabella, aggiunge degli zeri in cima 
    #  alla lista della riga, finchè non risulta della lunghezza 
    #  appropriata.
    def to_binary(self, num, l) :
        bin = "{0:b}".format(num)
        bin_list = list(bin)
        while (len(bin_list) < l) : 
            bin_list = ['0'] + bin_list
        return bin_list


    # Metodo che per ogni linea calcola la percentuale
    #  Essendo le linee riempite con delle liste di 0 e 1
    #   questo metodo scandisce ognuna di queste linee e 
    #   usa come valore da moltiplicare (in riferimento 
    #   alla lista di attributi tipical_attrs) 1-p (con p percentuale
    #   nella lista attr) se il valore incontrato è 0, p se il
    #   valore è 1.
    #  Una volta che ha moltiplicato tra loro tutti i valori della 
    #   linea, li moltiplica ancora per 100 e poi li aggiunge alla linea.
    def addPercentage(self) :
               
        for line in range(len(self.table)) :
            tot = 100
            for x in range(len(self.table[line])) :
                if (self.table[line][x] == '1') :
                    tot *= float(self.tipical_attrs[x][1]) 
                else :
                    tot *= 1-float(self.tipical_attrs[x][1])

            self.table[line] += [tot]


    # Metodo che restituisce a tabella ordinata usando il metodo 
    #  ausiliario SortTable() che implementa un algoritmo di quick sort.
    def getSortedTable(self) :
        return self.sortTable(self.table)


    # Metodo che stampa la tabella (ordinata)
    def printTable(self):
        i = len(self.sorted_table)-1
        while i >= 0:     
            #print(str(self.sorted_table[i])+'')
            i-=1

        
    def sortTable(self, table) :
        less, equal, greater = [], [], []

        if len(table) > 1:
            line_len = len(table[0])
            pivot = table[0][line_len - 1]
            for x in table :
                if x[line_len-1] < pivot :
                    less.append(x)
                if x[line_len-1] == pivot :
                    equal.append(x)
                if x[line_len-1] > pivot :
                    greater.append(x)

            return self.sortTable(less)+equal+self.sortTable(greater)
        
        else :
            return table


    # Metodo che richiama ManageOntology e per ogni 
    #  scenario (linea della tabella) controlla che sia consistente
    #   Per prima cosa cancella gli scenari che prendono 
    #     tutte le proprietà e quelli che non ne prendono nessuna, perchè banali
    #   Poi per ogni riga della tabella crea un'ontologia e controlla 
    #     che sia consistente, in caso di risposta negativa la riga viene cancellata
    def delNotConsistent(self) :
        i = 0
        del self.table[-1]
        del self.table[0]

        while i < len(self.table) :     
            ont = ManageOntology(tipical_attrs = self.tipical_attrs, attrs = self.attrs, line = self.table[i])
            consistent = ont.is_consistent()
            #print(str(consistent)+', '+str(self.table[i][:-1])+'\n')
            print('Scenario numero '+str(i)+': '+str(self.table[i])+'\n\n\n')
            if not consistent :
                del self.table[i]
                i -= 1
            i += 1
        
'''

    def getSignificants(self) :
                
        resulting_scen = []

                
'''
    

if __name__ == "__main__" :
    print("fatto")
#    dict_of_attr = DataFromWeb.DictOfAttr()
# =============================================================================
#     f = ReadOntology() 
#     pprint(f.ontology)
#     a = HeadModifier("martello", "arco", f.ontology)
#     tab = Table(a)
#     pprint(tab.table)
#     pprint(tab.tipical_attrs)
#     pprint(tab.attrs)
# =============================================================================





     
